"use strict";

var _require = require('../utils'),
    isReactComponent = _require.isReactComponent,
    importSpecifier = _require.importSpecifier,
    expr2str = _require.expr2str,
    ClassNames = _require.ClassNames,
    LibraryClassNames = _require.LibraryClassNames,
    existClassAttrName = _require.existClassAttrName,
    getImportSpecifier = _require.getImportSpecifier;

var options = require('../options');

module.exports = function (_ref) {
  var t = _ref.types,
      template = _ref.template;
  var scope = Boolean(options.scope);
  var classAttrs = options.classAttrs;

  function JSXAttributeVisitor(path) {
    var tagName = path.parent && expr2str(path.parent.name);
    var arrtName = expr2str(path.node.name);
    if (!classAttrs.some(function (classAttrName) {
      return existClassAttrName(classAttrName, arrtName, tagName);
    }) || !t.isJSXExpressionContainer(path.node.value)) return;
    var expression = path.node.value.expression;
    if (t.isStringLiteral(expression) || t.isTemplateLiteral(expression) || t.isCallExpression(expression) && expr2str(expression.callee) === this.CLASSNAMES) return;

    if (!this.libraryVarSpecifier) {
      this.libraryVarSpecifier = importSpecifier(path, "".concat(this.CLASSNAMES, ",default"), LibraryClassNames);
    }

    path.node.value.expression = template('$RCS($EXPR$)')({
      $RCS: this.libraryVarSpecifier.local.name,
      $EXPR$: expression
    }).expression;
  }

  return {
    visitor: {
      Program: {
        enter: function enter(path) {
          if (!scope || !isReactComponent(path)) return;
          var libraryVarSpecifier = getImportSpecifier(path, LibraryClassNames);
          var ctx = {
            libraryVarSpecifier: libraryVarSpecifier,
            CLASSNAMES: libraryVarSpecifier ? expr2str(libraryVarSpecifier.imported || libraryVarSpecifier.local) : ClassNames
          };
          path.traverse({
            JSXAttribute: JSXAttributeVisitor
          }, ctx);
        }
      }
    }
  };
};